;                         ****************************
;                         *        JOYPAD.MSG        *
;                         *       version 1.04       *
;                         *                          *
;                         *      17 / 04 / 1996      *
;                         ****************************

;  Source for Assemble.

fifo_size	equ	12
nkbd		equ	256	;nkbd bindings max. for the keyboard. Must be >= 1.
nkeys		equ	21
cr		equ	13
lf		equ	10
gemdos		equ	1
xbios		equ	14
cconout		equ	$02
cconws		equ	$09
fopen		equ	$3D
fclose		equ	$3E
fread		equ	$3F
fseek		equ	$42
seek_set	equ	0
seek_end	equ	2
malloc		equ	$48
mfree		equ	$49
supexec		equ	$26

handle		equr	d3
length		equr	d4
nline		equr	d5

iregs		reg	d0-d2/a0-a3
nregs		reg	d0/a0
sregs		reg	d1-d6/a0-a5
gregs0		reg	d0-d6/a0-a3
gregs		reg	d1-d6/a0-a3
rregs		reg	d0-d5/a0-a5

buff		RSSTRUCT		;Struct of the block that will be reserved.
		rs.l	fifo_size	;Queue (signals to be sent).			  \
fnc		rs.b	1		;0: joypad not active during high scores display. |
kbd_flag	rs.b	1		;0: keyboard signals are not taken into account.  |
menubind	rs.b	13		;Bindings for the main menu and game over.	  |
esc_state	rs.b	1		;Used in game_over (state of the button).	  |
mm_state	rs.b	8		;Used in main_menu (state of the buttons).	  |
gamebind	rs.l	2*nkeys		;Bindings for the game (2 * 21 keys).		  |
active		rs.b	1		;0: non active, -1: active.			  /
state		rs.b	2*nkeys		;State of the keys (0: released, -1: pressed).
		rseven.l
keys		rs.w	3*nkbd		;Bindings of the keys of the *keyboard*.
		rs.w	1		;0.
		rseven.l
		RSEND

mgemdos        MACRO
               move.w   #\1,-(sp)
               trap     #gemdos
               add.l    #\2,sp
               ENDM

               TEXT

header:		ascii	'MSG1'			;Header.
		pea	([header.w,pc])		;Address of the next message file.
		cmpi.w	#$0005,d0
		bne.s	lbl2
		move.l	a0,-(sp)
		movea.l	buffer(pc),a0		;Clear mm_state before the first
		clr.l	buff.mm_state(a0)	;main_menu call (i.e. keys supposed
		clr.l	buff.mm_state+4(a0)	;to be pressed).
		movea.l	(sp)+,a0
		rts
lbl2:		cmpi.w	#$0202,d0
		bne.s	lbl3
		move.l	a0,-(sp)
		movea.l	buffer(pc),a0		;Clear esc_state before the first game_over
		clr.b	buff.esc_state(a0)	;call (i.e. key supposed to be pressed).
		movea.l	(sp)+,a0
		rts
lbl3:		cmpi.w	#$0203,d0
		beq.s	clr_fifo
		cmpi.w	#$0300,d0
		beq.s	send_signal
		cmpi.w	#$0302,d0
		beq	main_menu
		cmpi.w	#$0304,d0
		beq	game_over
		cmpi.w	#$0007,d0
		beq	new_rule
		cmpi.w	#$0001,d0
		bne.s	rts1
		movem.l	iregs,-(sp)		;Init.
		lea	errflag(pc),a3
		move.b	6(a0),(a3)		;Error display flag.
		move.l	#buff.SIZEOF,-(sp)
		mgemdos	malloc,6		;Reserve a memory block.
		lea	buffer(pc),a0
		move.l	d0,(a0)			;Store the address to buffer.
		beq.s	error
		movea.l	d0,a0
		move.w	#buff.SIZEOF/4-1,d0
init_loop:	clr.l	(a0)+			;Clear the block.
		dbf	d0,init_loop
		movem.l	(sp)+,iregs
rts1:		rts

error:		tst.b	(a3)
		beq.s	error2			;Branch if errflag = 0.
		lea	insufmem(pc),a0
		bsr	disp_string		;Display "joypad: insufficient memory".
error2:		movem.l	(sp)+,iregs		;Insufficient memory.
		addq.l	#4,sp			;Don't go to the next message file.
		moveq	#1,d0			;Return code: 1.
		rts

clr_fifo:	movem.l	nregs,-(sp)		;New piece.
		movea.l	buffer(pc),a0
		moveq	#fifo_size-1,d0
clr_loop:	clr.l	(a0)+			;Clear the signal queue.
		dbf	d0,clr_loop
		movem.l	(sp)+,nregs
rts2:		rts

send_signal:	tst.b	([buffer.w,pc],buff.active.w)
		beq.s	rts2			;Branch if non active.
		movem.l	sregs,-(sp)
		movea.l	a0,a5
		pea	read_keys(pc)
		move.w	#supexec,-(sp)
		trap	#xbios			;Call read_keys in superuser mode.
		addq.l	#6,sp
		movea.l	buffer(pc),a0		;A0: signal queue.
		move.l	4(a5),d2		;\ Branch if no key (of the
		beq.s	no_kbdsig		;/ keyboard) has been hit.
		lsr.l	#8,d2			;D2.W: special keys and scancode.
		lea	buff.keys(a0),a1
		bra.s	keys_next
keys_loop:	cmp.w	d1,d2
		beq.s	keys_found		;Branch if binding matches.
keys_next:	move.w	(a1)+,d1		;Next key binding or 0.
		bne.s	keys_loop
		tst.b	buff.kbd_flag(a0)	;\ Go to the next message file if keyboard
		bne.s	sendsig_end		;/ signals must be taken into account.
no_kbdsig:	movea.l	a0,a1
		move.l	(a1)+,d0		;D0.L: head signal or 0 (empty queue).
		bne.s	shift
		moveq	#-1,d0			;Empty queue.
		tst.b	buff.kbd_flag(a0)	;\ Branch if keyboard signals mustn't
		beq.s	return			;/ be taken into account.
sendsig_end:	movem.l	(sp)+,sregs
		rts				;Go to the next message file.
keys_found:	move.l	(a1),d0			;D0.L: signal.
		bra.s	return
shift:		moveq	#fifo_size-2,d1		;Remove the head signal and return it.
shift_loop:	move.l	(a1)+,(a0)+
		dbf	d1,shift_loop
return:		movem.l	(sp)+,sregs
		addq.l	#4,sp
		rts				;Return to TETRHEX.

read_keys:	movea.l	buffer(pc),a0		;A0: signal queue.
		lea	buff.gamebind(a0),a1	;A1: game bindings.
		lea	buff.state(a0),a2	;A2: state of the keys.
		lea	cmds(pc),a3		;A3: operations to be performed.
		lea	$FF9202,a4		;A4 and A4-2: addresses for the joypad.
		moveq	#fifo_size-1,d0
rdkeys_loop1:	tst.l	(a0)+			;Seek the tail of the signal queue.
		dbeq	d0,rdkeys_loop1
		bne.s	read_keys2		;Branch if the signal queue is full.
		subq.l	#4,a0			;A0: address of the next empty cell.
read_keys2:	moveq	#7,d0			;8 masks are to be used.
rdkeys_loop2:	move.w	(a3)+,d6		;D6.W: mask (to read some keys).
		clr.w	d3			;D3.W; will be 0 and -2.
rdkeys_outer:	move.w	d6,(a4)			;Write the mask.
		move.w	(a4,d3.w),d1		;Read some keys.
		move.b	(a3)+,d2		;\ The state of the keys are in the
		lsr.w	d2,d1			;/ less significant bits of D1.
		move.b	(a3)+,d2		;D2.B: number of read keys - 1.
rdkeys_inner:	lsr.w	#1,d1			;C: state of the key (0: pressed).
		scc	d4			;D4.B: 0: released, -1: pressed.
		bcs.s	rdkeys_next		;Branch if the key is released.
		tst.b	(a2)
		bne.s	rdkeys_next		;Branch if the key was already pressed.
		tst.l	(a1)
		beq.s	rdkeys_next		;Branch if there's no associated signal.
		moveq	#fifo_size*4,d5
		add.l	buffer(pc),d5
		cmp.l	a0,d5
		beq.s	rdkeys_next		;Branch if the signal queue is full.
		move.l	(a1),(a0)+		;Store the signal to the tail.
rdkeys_next:	addq.l	#4,a1			;Address of the next binding.
		move.b	d4,(a2)+		;Store the new state.
		subq.b	#1,d2
		bcc.s	rdkeys_inner		;Next key in the same word...
		subq.w	#2,d3
		bcs.s	rdkeys_outer		;Branch if new D3.W = -2.
		dbf	d0,rdkeys_loop2		;Next mask...
rts4:		rts

game_over:	tst.b	([buffer.w,pc],buff.active.w)
		beq.s	rts4			;Go to the next message file if non active.
		movem.l	gregs0,-(sp)
		moveq	#0,d6
		bsr.s	test_key
		movem.l	(sp)+,gregs0
		rts				;Go to the next message file.

su_test_key:	lea	$FF9202,a0
		move.w	d4,(a0)			;Write the mask.
		move.w	(a0,d5.w),d0		;Answer.
rts6:		rts

test_key:	lea	cmds(pc),a0
		movea.l	buffer(pc),a3
		move.b	buff.menubind(a3,d6.l),d0  ;Joypad key identifier.
		beq.s	rts6			;Return if no key binding.
		lsr.b	#1,d0			;Joypad A: LSB = 0. Joypad B: LSB = 1.
		bcc.s	tstesc_outer
		lea	24(a0),a0
		bra.s	tstesc_outer
tstesc_loop:	subq.w	#2,d5			;\
		bcs.s	tstesc_inner		;|
tstesc_outer:	move.w	(a0)+,d4		;| D4.W: mask that will be written to read
		clr.w	d5			;| the key state. D5.W: 0 or -2 (indicates
tstesc_inner:	move.w	(a0)+,d3		;| the word that will be read). D3.W:
		addq.b	#1,d3			;| information about the bit of the
		sub.b	d3,d0			;| previous word that will be tested.
		bcc.s	tstesc_loop		;|
		add.b	d0,d3			;/
		pea	su_test_key(pc)
		move.w	#supexec,-(sp)
		trap	#xbios			;Call su_test_key in superuser mode.
		addq.l	#6,sp
		lsr.w	d3,d0
		lsr.w	#8,d3
		lsr.w	d3,d0
		andi.b	#1,d0			;D0.B = 0: key pressed. D0.B = 1: key released.
		cmp.b	buff.esc_state(a3,d6.l),d0
		beq.s	rts5			;Return if the key state hasn't changed.
		move.b	d0,buff.esc_state(a3,d6.l)  ;Store the new state.
		bne.s	rts5			;Return if the key is released.
		move.l	d6,d0
		addq.l	#8,sp			;Pull the return address and D0.
		movem.l	(sp)+,gregs		;Restore the other registers.
		addq.l	#4,sp			;Pull the address of the next message file.
		beq.s	add			;Branch if D0 = 0 (game_over). --> D0 != 0.
		cmp2.l	limits(pc),d0		;D0 in [9..12]?
		bcs.s	rts5			;Branch if not "change the size of the board".
add:		addq.b	#7,d0
rts5:		rts				;Return to TETRHEX.

main_menu:	tst.b	([buffer.w,pc],buff.active.w)
		beq.s	rts5			;Go to the next message file if non active.
		movem.l	gregs0,-(sp)
		moveq	#12,d6
mm_loop:	bsr.s	test_key
		subq.b	#1,d6
		bne.s	mm_loop
		movem.l	(sp)+,gregs0
		rts				;Go to the next message file.

seek:		move.w	d0,-(sp)
		move.w	handle,-(sp)
		clr.l	-(sp)
		mgemdos	fseek,10
		tst.l	d0
		rts

new_rule:	movem.l	rregs,-(sp)
		lea	([4.w,a0],22.w),a4	;A4: rule name.
		movea.l	buffer(pc),a0
		move.w	#buff.SIZEOF/4-1,d0
nrule_clr:	clr.l	(a0)+			;Clear the block.
		dbf	d0,nrule_clr
		clr.w	-(sp)
		pea	fname(pc)
		mgemdos	fopen,8			;Open "JOYPAD.CFG".
		lea	open_error(pc),a0
		move.l	d0,handle
		bmi.s	nrule_error
		moveq	#seek_end,d0
		bsr.s	seek
		lea	seek_error(pc),a5
		bmi.s	nrule_close
		lea	zero_error(pc),a5
		beq.s	nrule_close
		move.l	d0,length		;--> length of the file.
		addq.l	#1,d0
		move.l	d0,-(sp)
		mgemdos	malloc,6		;Reserve a block for the file + $00.
		lea	insufmem(pc),a5
		tst.l	d0
		bmi.s	nrule_close
		movea.l	d0,a3			;A3: address of the block.
		move.l	d0,-(sp)		;Push the address (for mfree, later).
		moveq	#seek_set,d0
		bsr.s	seek			;File pointer: 0.
		lea	seek_error(pc),a5
		bmi.s	nrule_free
		move.l	length,-(sp)
		move.w	handle,-(sp)
		mgemdos	fread,8			;Read the whole file.
		cmp.l	d0,length
		beq	nrule_noerr
		lea	read_error(pc),a5
nrule_free:	mgemdos	mfree,6			;Error...
nrule_close:	move.w	handle,-(sp)
		mgemdos	fclose,4
		movea.l	a5,a0
nrule_error:	tst.b	errflag(pc)
		beq.s	nrule_end
		bsr	disp_string		;Display the error message.
nrule_end:	movem.l	(sp)+,rregs
		rts

next_line2:	movea.l	a3,a0			;A0: first non-blank character.
eol_loop:	tst.b	(a3)			;Seek the end of line.
		beq.s	eol			;Branch if end of file.
		cmp.b	(a3),d1
		bne.s	eol_tstlf		;Branch if not CR.
		clr.b	(a3)+			;End of line.
		cmp.b	(a3)+,d2		;Compare the next character to LF.
		beq.s	eol			;Branch if CR/LF (--> MS-DOS).
		subq.l	#1,a3			;It was CR (--> Mac).
		bra.s	eol
eol_tstlf:	cmp.b	(a3)+,d2		;Compare to LF (--> Unix?).
		bne.s	eol_loop		;Branch if not LF.
		clr.b	-1(a3)			;End of line.

; Note: $00 indicates the end of line. A3 points to the next line, i.e. the character
;       following $00, or $00 if this byte also indicates the end of file.

eol:		movea.l	a0,a1
		cmpi.b	#'B',(a1)+
		bne.s	not_begin
		cmpi.b	#'E',(a1)+
		bne.s	normal_line
		cmpi.b	#'G',(a1)+
		bne.s	normal_line
		cmpi.b	#'I',(a1)+
		bne.s	normal_line
		cmpi.b	#'N',(a1)+
		bne.s	normal_line
		tst.b	(a1)
		beq.s	begin
normal_line:	clr.b	d0			;Normal line: N = 0, Z = 1, C = 0.
		rts
begin:		moveq	#1,d0			;BEGIN: N = 0, Z = 0, C = 0.
		rts
not_begin:	cmpi.b	#'E',(a0)
		bne.s	normal_line
		cmpi.b	#'N',(a1)+
		bne.s	normal_line
		cmpi.b	#'D',(a1)+
		bne.s	normal_line
		tst.b	(a1)
		bne.s	normal_line
		moveq	#-127,d0
		add.b	d0,d0			;END: N = 0, Z = 0, C = 1.
		rts

next_line:	moveq	#$20,d0			;Seek the next nonempty line.
		moveq	#cr,d1
		moveq	#lf,d2
inc_nline:	addq.l	#1,nline
bol_loop:	cmp.b	(a3),d0
		bcs.s	next_line2
		cmp.b	(a3)+,d1
		bne.s	lbl1
		cmp.b	(a3)+,d2
		beq.s	inc_nline
lbl1:		cmp.b	-1(a3),d2
		beq.s	inc_nline
		tst.b	-1(a3)
		bne.s	bol_loop
		moveq	#-1,d0			;No line any more: N = 1, Z = 0, C = 0.
		rts				;Null character: end of file.

set_active:	st	buff.active(a2)		;Set active.
free:		mgemdos	mfree,6			;Free the memory block.
		movem.l	(sp)+,rregs
		rts

nrule_noerr:	move.w	handle,-(sp)
		mgemdos	fclose,4		;Close the file.
		clr.b	(a3,length.l)		;Store $00 at the end of the file.
		moveq	#0,nline		;Initialize nline (line number).
		movea.l	buffer(pc),a2
next_block:	bsr.s	next_line		;First line of the block...
		bmi.s	set_active		;Branch if end of file. No error.
		bne.s	block_ok		;Branch if there's no rule name.
next_name:	movea.l	a4,a1			;Compare the names.
cmp_names:	move.b	(a0)+,d0
		beq.s	cmp_names2
		cmp.b	(a1)+,d0
		beq.s	cmp_names
different:	bsr.s	next_line		;The names are different.
		bmi.s	prg_end			;Error if end of file.
		beq.s	next_name		;Branch if normal line.
		bcs.s	prg_end			;Error if END (instead of BEGIN).
sk_end:		bsr.s	next_line		;Seek END.
		beq.s	sk_end			;Loop if normal line.
		bcs.s	next_block		;Branch if END (--> next block).
prg_end:	bsr	disp_error
		bra.s	free
cmp_names2:	tst.b	(a1)
		bne.s	different		;Branch unless the names are the same.
sk_begin:	bsr.s	next_line		;Seek BEGIN.
		beq.s	sk_begin		;Loop if normal line.
		bmi.s	prg_end			;Error if end of file.
block_ok:	bcs.s	prg_end			;Error if END (instead of BEGIN).
next_bind:	bsr.s	next_line		;Next binding or END.
		bcs.s	next_block		;Branch if END.
		bne.s	prg_end			;Error if BEGIN or end of file.
		move.b	(a0)+,d2		;First non-blank character.
		subi.b	#'B',d2
		bhi	cmd_kbd			;Branch if > 'B' (it must be a 'K').
		addq.b	#1,d2
		bmi	cmd_kbdkey		;Branch if it is neither 'A' nor 'B'.
		move.b	(a0)+,d0		;Second character (indicates the key).
		lea	keys(pc),a1		;List of the admissible characters.
		moveq	#nkeys-1,d1
cmpkey_loop:	cmp.b	(a1)+,d0
		dbeq	d1,cmpkey_loop		;--> D1.W: number of the key (< nkeys).
		bne.s	prg_end			;Error if bad character.
		bsr	skip_blanks		;Skip the blanks (error if end of file).
		move.b	(a0)+,d0		;Next character.
		cmpi.b	#'"',d0
		bne.s	menu_bind		;Branch if the next character isn't '"'.
		tst.b	d2
		beq.s	st_signame1
		addi.b	#nkeys,d1
st_signame1:	lea	(buff.gamebind.w,a2,d1.w*4),a1
st_signame2:	move.b	(a0)+,(a1)+		;\
		beq.s	prg_end			;|
		move.b	(a0)+,(a1)+		;|
		beq.s	prg_end			;| Store the signal
		move.b	(a0)+,(a1)+		;| name.
		beq.s	prg_end			;|
		move.b	(a0)+,(a1)		;|
		beq.s	prg_end			;/
		cmpi.b	#'"',(a0)+
		bne.s	prg_end			;Error if the next character isn't '"'.
end_bind:	tst.b	(a0)
		beq.s	next_bind		;Loop if it was the last character.
prg_end2:	bra.s	prg_end			;Otherwise, error.

menu_bind:	cmpi.b	#'.',d0
		beq.s	menu_bind2		;Branch if it is a period.
		tst.b	(a0)
		bne.s	prg_end			;Error if there are other characters.
		subi.b	#'8',d0			;\ Error if it isn't a digit
		bhi.s	prg_end			;| between 0 and 8. Otherwise
menu_bind3:	addq.b	#8,d0			;| D0.B: between 0 and 8
		bmi.s	prg_end2		;/ (item identifier).
		lsr.b	#1,d2			;Joypad A: C = 0. Joypad B: C = 1.
		roxl.b	#1,d1			;D1.B: key identifier.
		ext.w	d0			;D0.W: item identifier.
		beq.s	store_menub		;Branch. if it is the game over item.
		moveq	#11,d2
menub_loop:	cmp.b	buff.menubind+1(a2,d2.l),d1
		bne.s	dbf
		clr.b	buff.menubind+1(a2,d2.l)  ;Remove the duplicate key identifiers.
dbf:		dbf	d2,menub_loop
store_menub:	move.b	d1,buff.menubind(a2,d0.w)  ;Store the key identifier for the item.
		bra	next_bind

menu_bind2:	move.b	(a0)+,d0		;Following character.
		tst.b	(a0)
		bne.s	prg_end2		;Error if there are other characters.
		subi.b	#'9',d0			;\
		bhi.s	prg_end2		;| Error if it isn't a digit between 6 and 9.
		addq.b	#3,d0			;| Otherwise D0.B: between 0 and 3.
		bmi.s	prg_end2		;/
		addq.b	#1,d0			;\ D0.B: between 1 and 4 (will
		bra.s	menu_bind3		;/ be between 9 and 12).

cmd_kbd:	cmpi.b	#'K'-'B',d2		;\
		bne.s	prg_end2		;|
		cmpi.b	#'B',(a0)+		;| The first 3 characters
		bne.s	prg_end2		;| must be 'KBD'.
		cmpi.b	#'D',(a0)+		;|
		bne.s	prg_end2		;/
		bsr.s	skip_blanks		;Skip the blanks (error if end of file).
		move.b	(a0)+,d0		;\
		subi.b	#'0',d0			;| The next character must
		lsr.b	#1,d0			;| be '0' or '1'.
		bne.s	prg_end2		;/
		scs	buff.kbd_flag(a2)	;Store the flag: '0': 0, '1': -1.
		bra.s	end_bind

skip_blanks:	cmp.b	#$20,(a0)		;Skip the blanks.
		bhi.s	rts3
		tst.b	(a0)+
		bne.s	skip_blanks
		addq.l	#4,sp			;End of file: error.
		bra.s	prg_end2

disp_error:	tst.b	errflag(pc)
		beq.s	rts3			;Branch if errflag = 0.
		lea	errline(pc),a0
		bsr.s	disp_string		;Display "joypad: error at line ".
		moveq	#10,d0
		moveq	#-1,d1
		bsr.s	disp_nline		;Display the line number.
		lea	crlf(pc),a0		;Display a newline...
disp_string:	pea	(a0)
		move.w	#cconws,-(sp)
		trap	#gemdos
		addq.l	#6,sp
rts3:		rts

disp_nline:	move.w	d1,-(sp)		;Display nline in decimal.
		divul.l	d0,d1:nline
		tst.l	nline
		bne.s	disp_nline
dnl_loop:	ori.b	#'0',d1
		move.w	d1,-(sp)
		move.w	#cconout,-(sp)
		trap	#gemdos
		addq.l	#4,sp
		move.w	(sp)+,d1
		bpl.s	dnl_loop
		rts

cmd_kbdkey:	cmpi.b	#'$'-'A',d2		;Error if the 1st character is not a '$'.
		bne.s	prg_end3
		moveq	#3,d2			;1st character: '$'.
kbdkey_loop1:	lsl.w	#4,d1
		move.b	(a0)+,d0		;Read the next character. Must be a hex digit.
		subi.b	#'0',d0
		bcs.s	prg_end3
		cmpi.b	#9,d0
		bls.s	kbdkey_lbl1
		cmpi.b	#'A'-'0',d0
		bcs.s	prg_end3
		cmpi.b	#'F'-'0',d0
		bhi.s	prg_end3
		subq.b	#'A'-'0'-10,d0
kbdkey_lbl1:	or.b	d0,d1
		dbf	d2,kbdkey_loop1
		tst.w	d1			;D1.W: scancode.
		beq.s	prg_end3		;Error if zero.
		lea	buff.keys(a2),a1	;A1: first binding.
		move.w	#nkbd-1,d2
kbdkey_loop2:	move.w	(a1),d0
		beq.s	kbdkey_lbl2		;New binding...
		cmp.w	d0,d1
		beq.s	kbdkey_lbl2		;Change the binding...
		addq.l	#6,a1
		dbf	d2,kbdkey_loop2
prg_end3:	bra	prg_end			;Error: too many bindings.
kbdkey_lbl2:	move.w	d1,(a1)+
		bsr	skip_blanks		;Skip the blanks (error if end of file).
		move.b	(a0)+,d0		;Next character.
		cmpi.b	#'"',d0
		bne.s	prg_end3		;Error if not '"'.
		bra	st_signame2		;Store the signal name.

buffer:		dc.l	0
limits:		dc.l	9,12
cmds:		dc.w	$FFFE,$803,$001,$FFFD,$803,$100,$FFFB,$803,$100,$FFF7,$803,$100
		dc.w	$FFEF,$C03,$201,$FFDF,$C03,$300,$FFBF,$C03,$300,$FF7F,$C03,$300
errflag:	dc.b	0
keys:		ascii	"O369#C2580B147*APRLDU"
fname:		asciiz	"JOYPAD.CFG"
insufmem:	ascii	"joypad: insufficient memory"
crlf:		dc.b	13,10,0
errline:	asciiz	"joypad: error at line "
open_error:	dc.b	"joypad: can't open JOYPAD.CFG",13,10,0
seek_error:	dc.b	"joypad: seek error",13,10,0
zero_error:	dc.b	"joypad: JOYPAD.CFG has a zero length",13,10,0
read_error:	dc.b	"joypad: can't read JOYPAD.CFG",13,10,0

		END
